/*
 * Created on Apr 10, 2005
 *
 */
package com.ibm.di.connector;

import java.util.HashMap;
import java.util.Vector;

import com.google.soap.search.GoogleSearch;
import com.google.soap.search.GoogleSearchFault;
import com.google.soap.search.GoogleSearchResult;
import com.google.soap.search.GoogleSearchResultElement;
import com.ibm.di.config.interfaces.ConnectorConfig;
import com.ibm.di.entry.Entry;

/**
 * @author Rudy Sutjiato
 *
 */
public class GoogleConnector extends Connector implements ConnectorInterface {
	private static final String CONNECTOR_NAME = "Google Search Connector";

	private static final String GOOGLE_LICENSE_KEY_PARAM = "google.Key";
	private static final String GOOGLE_QUERY_STRING_PARAM =
		"google.QueryString";
	private static final String GOOGLE_SAFE_SEARCH = "google.SafeSearch";
	private static final String GOOGLE_FILTER_RESULT =
		"google.FilterSimilarResult";
	private static final String GOOGLE_MAX_RESULTS = "google.MaxResults";
	private static final String GOOGLE_LANGUAGE = "google.Language";

	private static final String SCHEMA_TITLE = "title";
	private static final String SCHEMA_URL = "url";
	private static final String SCHEMA_SNIPPET = "snippet";
	private static final String SCHEMA_SUMMARY = "summary";

	private static String[][] metadata =
		{ { SCHEMA_TITLE, "java.lang.String", "256" }, {
			SCHEMA_URL, "java.lang.String", "1024" }, {
			SCHEMA_SNIPPET, "java.lang.String", "1024" }, {
			SCHEMA_SUMMARY, "java.lang.String", "2048" }
	};

	private GoogleSearch googleSearch = null;
	private GoogleSearchResult searchResult = null;
	private GoogleSearchResultElement searchResultElement[] = null;
	private int resultElementIndex = 0;
	private String licenseKey = null;
	private String queryString = null;
	private boolean safeSearch = true;
	private boolean filterSimilarResult = true;
	private int maxResults = 0;
	private String language = null;
	private static HashMap languageMap = null;

	public GoogleConnector() {
		setName(CONNECTOR_NAME);
		setModes(new String[] { ConnectorConfig.ITERATOR_MODE });
	}

	public String getVersion() {
		return "0.1 10 April 2005";
	}

	public void initialize(Object arg0) throws Exception {
		String safeSearchParam = null;
		String filterSimilarResultParam = null;

		debug("Initialize Google Connector");

		licenseKey = getParam(GOOGLE_LICENSE_KEY_PARAM);
		checkRequiredParameter(GOOGLE_LICENSE_KEY_PARAM, licenseKey);

		queryString = getParam(GOOGLE_QUERY_STRING_PARAM);
		checkRequiredParameter(GOOGLE_QUERY_STRING_PARAM, queryString);

		googleSearch = new GoogleSearch();
		googleSearch.setKey(licenseKey);
		googleSearch.setQueryString(queryString);

		safeSearchParam = getParam(GOOGLE_SAFE_SEARCH);
		if (safeSearchParam.equalsIgnoreCase("true")) {
			safeSearch = true;
		} else {
			safeSearch = false;
		}
		googleSearch.setSafeSearch(safeSearch);
		debug("Safe Search = " + safeSearch);

		filterSimilarResultParam = getParam(GOOGLE_FILTER_RESULT);
		if (filterSimilarResultParam.equalsIgnoreCase("true")) {
			filterSimilarResult = true;
		} else {
			filterSimilarResult = false;
		}
		googleSearch.setFilter(filterSimilarResult);
		debug("Filter Similar Result = " + filterSimilarResult);

		try {
			maxResults = Integer.parseInt(getParam(GOOGLE_MAX_RESULTS));
			googleSearch.setMaxResults(maxResults);
			debug("Maximum Search Result = " + maxResults);
		} catch (NumberFormatException e) {
			logmsg("Error: Maximum result parameter should be a number");
		}

		language = getParam(GOOGLE_LANGUAGE);
		if (language != null) {
			languageMap = new HashMap();
			populateLanguageMap();
			language = (String) languageMap.get(language);
			googleSearch.setLanguageRestricts(language);
			debug("Language = " + language);
		}
	}

	public void selectEntries() throws Exception {
		debug("Executing selectEntries() to do the search.");
		try {
			searchResult = googleSearch.doSearch();
		} catch (GoogleSearchFault e) {
			e.getCause();
			throw e;
		}
		searchResultElement = searchResult.getResultElements();
	}

	public Entry getNextEntry() throws Exception {
		Entry entry = null;
		if ((searchResultElement.length > 0)
			&& (resultElementIndex < searchResultElement.length)) {
			entry = new Entry();
			entry.setAttribute(
				SCHEMA_TITLE,
				searchResultElement[resultElementIndex].getTitle());
			entry.setAttribute(
				SCHEMA_URL,
				searchResultElement[resultElementIndex].getURL());
			entry.setAttribute(
				SCHEMA_SNIPPET,
				searchResultElement[resultElementIndex].getSnippet());
			entry.setAttribute(
				SCHEMA_SUMMARY,
				searchResultElement[resultElementIndex].getSummary());
			resultElementIndex++;
			return entry;
		} else {
			return null;
		}
	}

	public Object querySchema(Object schemaSource) throws Exception {
		Vector schema = new Vector();
		Entry entry = null;

		for (int i = 0; i < metadata.length; i++) {
			entry = new Entry();
			entry.addAttributeValue("name", metadata[i][0]);
			entry.addAttributeValue("syntax", metadata[i][1]);
			entry.addAttributeValue("size", metadata[i][2]);
			schema.add(i, entry);
		}
		return schema;
	}

	public boolean isDeltaSupported() {
		return false;
	}

	private void checkRequiredParameter(String paramName, String paramValue)
		throws Exception {
		String errorMsg = null;
		if ((paramValue == null) || (paramValue.trim().equals(""))) {
			errorMsg = "Error: Missing required parameter " + paramName;
			logmsg(errorMsg);
			throw new Exception(errorMsg);
		}
	}

	private void populateLanguageMap() {
		languageMap.put("Arabic", "lang_ar");
		languageMap.put("Chinese Simplified", "lang_zh-CN");
		languageMap.put("Chinese Traditional", "lang_zh-TW");
		languageMap.put("Czech", "lang_cs");
		languageMap.put("Danish", "lang_da");
		languageMap.put("Dutch", "lang_nl");
		languageMap.put("English", "lang_en");
		languageMap.put("Estonian", "lang_et");
		languageMap.put("Finnish", "lang_fi");
		languageMap.put("French", "lang_fr");
		languageMap.put("German", "lang_de");
		languageMap.put("Greek", "lang_el");
		languageMap.put("Hebrew", "lang_iw");
		languageMap.put("Hungarian", "lang_hu");
		languageMap.put("Icelandic", "lang_is");
		languageMap.put("Italian", "lang_it");
		languageMap.put("Japanese", "lang_ja");
		languageMap.put("Korean", "lang_ko");
		languageMap.put("Latvian", "lang_lv");
		languageMap.put("Lithuanian", "lang_lt");
		languageMap.put("Norwegian", "lang_no");
		languageMap.put("Portuguese", "lang_pt");
		languageMap.put("Polish", "lang_pl");
		languageMap.put("Romanian", "lang_ro");
		languageMap.put("Russian", "lang_ru");
		languageMap.put("Spanish", "lang_es");
		languageMap.put("Swedish", "lang_sv");
		languageMap.put("Turkish", "lang_tr");
	}

	public void reconnect(Object o) throws Exception {
		terminate();
		initialize(o);
	}

	public void terminate() throws Exception {
		searchResult = null;
		googleSearch = null;
	}
}
